from typing import Any, Dict


def obtain_search_keywords() -> Dict[str, Any]:
    """Obtain search keywords dictionary.

    Returns:
        Dict[str, Any]: Dictionary containing categorized search keywords.
    """
    _h_ = "(?:| |-)"  # hyphen

    evol = "evol(?:ution|utionary)"  # 'evol(?:ution|utionary|ve|ved|ving)'
    computation = "computation(?:|al)"
    strateg = "strateg(?:y|ies)"
    program = "program(?:|ming)"
    algorithm = "algorithm(?:|s)"
    automat = "automat(?:ed|ion)"
    keywords_ec = [  # evolution computation
        ["simulated annealing"],
        ["taboo search"],
        [f"{evol} {strateg}"],
        ["CMA-ES"],  #
        [f"{evol} {program}"],
        [f"differential {evol}"],
        [f"{evol} {algorithm}"],
        [[evol], [strateg, program, "differential", algorithm]],
        [f"genetic {algorithm}"],
        [f"genetic {program}"],
        [["genetic"], [algorithm, program]],
        ["particle swarm"],
        [["swarm"], ["particle"]],
        ["ant colony"],
        ["bee colony"],
        [["colony"], ["ant", "bee"]],
        [f"memetic {algorithm}"],
        [f"population{_h_}based"],
        ["quality diversity"],
        [evol, algorithm, automat],
        [evol, computation],
    ]

    keywords_ss = [  # search strategy
        ["local search"],
        [["local", "search"], ["local search"]],
        ["local optimization"],
        [["local", "optimization"], ["local optimization"]],
        ["random search"],
        [["random", "search"], ["random search"]],
        ["random optimization"],
        [["random", "optimization"], ["random optimization"]],
        ["global search"],
        [["global", "search"], ["global search"]],
        ["global optimization"],
        [["global", "optimization"], ["global optimization"]],
        ["heuristic search"],
        [["heuristic", "search"], ["heuristic search"]],
        ["heuristic optimization"],
        [["heuristic", "optimization"], ["heuristic optimization"]],
    ]

    nsga = "NSGA(?:|II|-II|III|-III)"
    moea_d = "MOEA/D"
    network = "network(?:|s)"
    uncertain = "uncertain(?:|ty)"
    keywords_multi = [  # multi objective
        [moea_d],
        [nsga],
        [f"multi{_h_}objective optimization"],
        [[f"multi{_h_}objective", "optimization"], [f"multi{_h_}objective optimization"]],
        [[f"multi{_h_}objective"], ["optimization"]],
        [f"multi{_h_}model optimization"],
        [[f"multi{_h_}model", "optimization"], [f"multi{_h_}model optimization"]],
        [[f"multi{_h_}model"], ["optimization"]],
        [f"many{_h_}objective optimization"],
        [[f"many{_h_}objective", "optimization"], [f"many{_h_}objective optimization"]],
        [[f"many{_h_}objective"], ["optimization"]],
        [f"dynamic multi{_h_}objective"],
        [f"dynamic {evol} multi{_h_}objective"],
        [["dynamic", f"multi{_h_}objective"], [f"dynamic multi{_h_}objective", f"dynamic {evol} multi{_h_}objective"]],
        [f"dynamic multi{_h_}model"],
        [["dynamic", f"multi{_h_}model"], [f"dynamic multi{_h_}model"]],
        [f"dynamic many{_h_}objective"],
        [f"dynamic {evol} many{_h_}objective"],
        [["dynamic", f"many{_h_}objective"], [f"dynamic many{_h_}objective", f"dynamic {evol} many{_h_}objective"]],
        ["dynamic", "optimization"],
        ["dynamic", network],
        [["dynamic"], [f"multi{_h_}objective", f"multi{_h_}model", f"many{_h_}objective", "optimization", network]],
        [f"{uncertain} optimization"],
        [[uncertain, "optimization"], [f"{uncertain} optimization"]],
        [[uncertain], ["optimization"]],
        ["pareto optimization"],
        [["pareto", "optimization"], ["pareto optimization"]],
        [["pareto"], ["optimization"]],
    ]

    dimension = "dimension(?:|al)"
    distribut = "distribut(?:ion|ed)"
    keywords_parallel = [  # parallel
        [f"large{_h_}scale"],
        [f"high{_h_}{dimension}"],
        [f"high{_h_}performance"],
        ["parallel", evol],
        ["parallel", algorithm],
        [["parallel"], [evol, algorithm]],
        [distribut, evol],
        [distribut, algorithm],
        [[distribut], [evol, algorithm]],
    ]

    keywords_mo = [  # math optimization
        [f"zero{_h_}orde", "optimization"],
        ["coordinate", "descent"],
        ["gradient", "descent"],
        ["gradient", "stochastic"],
        [["gradient"], ["descent", "stochastic"]],
        ["convex", "optimization"],
        [f"non{_h_}convex", "optimization"],
        [["convex"], [f"non{_h_}convex", "optimization"]],
        [[f"non{_h_}convex"], ["convex", "optimization"]],
        ["stochastic", "optimization"],
        [["stochastic"], ["optimization"]],
        ["gaussian", "distribution"],
    ]

    multi_task = "multi(?:|-)task"
    federa = "federa(?:l|ted)"
    weakly_ = f"weakly{_h_}"
    generat = "generat(?:ive|ion)"
    keywords_ml = [  # machine learning
        ["automated", "machine", "learning"],
        [["machine", "learning"], [automat]],
        ["deep", "learning"],
        [f"semi{_h_}supervised", "learning"],
        [f"self{_h_}supervised", "learning"],
        [f"{weakly_}supervised", "learning"],
        ["unsupervised", "learning"],
        [f"multi{_h_}instance", "learning"],
        ["active", "learning"],
        [
            ["supervised", "learning"],
            [f"semi{_h_}supervised", f"self{_h_}supervised", f"weakly{_h_}supervised", "unsupervised"],
        ],
        ["reinforcement", "learning", f"on{_h_}policy"],
        ["reinforcement", "learning", f"off{_h_}policy"],
        ["reinforcement", "learning", "offline"],
        ["reinforcement", "learning", f"model{_h_}based"],
        ["reinforcement", "learning", "continual"],
        ["reinforcement", "learning", "deep"],
        ["reinforcement", "learning", evol],
        [
            ["reinforcement", "learning"],
            ["offline", f"on{_h_}policy", f"off{_h_}policy", f"model{_h_}based", "deep", "continual", evol],
        ],
        ["policy", "search"],
        [["policy"], ["policy", "search"]],
        [f"q{_h_}learning"],
        ["manifold", "learning"],
        [["manifold"], ["Learning"]],
        [multi_task, "learning"],
        [[multi_task], ["learning"]],
        ["transfe", "learning"],
        [["transfe"], ["Learning"]],
        ["domain", "adaptation"],
        ["domain", "generalization"],
        [f"meta{_h_}learning"],
        [[f"meta{_h_}learning"], ["learning"]],
        [federa, "learning"],
        [[federa], ["learning"]],
        ["ensemble", "learning"],
        [["ensemble"], ["learning"]],
        ["online", "learning"],
        [f"few{_h_}shot", "learning"],
        [[f"few{_h_}shot"], ["learning"]],
        [f"one{_h_}shot", "learning"],
        [[f"one{_h_}shot"], ["learning"]],
        [f"zero{_h_}shot", "learning"],
        [[f"zero{_h_}shot"], ["learning"]],
        ["representation", "learning"],
        [["representation"], ["learning"]],
        ["induction"],
        ["deduction"],
        ["transduction"],
        ["neural", network],
        ["graph", network],
        [[network], ["graph", "neural"]],
        [["graph"], [network, "neural"]],
        ["kernel"],
        ["embedding"],
        ["transformer"],
        ["diffusion", "model"],
        [["diffusion"], ["model"]],
        [generat, "model"],
        [[generat], ["model"]],
        ["large language model"],
        [["large", "language", "model"], ["large language model"]],
    ]

    cluster = "cluster(?:|s|ing)"
    data_driven = "date(?:| |-)driven"
    prove = "prov(?:able|e)"
    predict = "predict(?:|ed|ion)"
    recommend = "recommend(?:ed|ation)"
    markov = "markov(?:|ian)"
    keywords_ec_ml = [  # evolution computation and machine learning
        ["neuro(?:| |-)evolution"],
        ["adaptation"],
        ["bayesian", "optimization"],
        ["bi-level", "optimization"],
        ["bayesian", "inference"],
        ["bayesian", "learning"],
        [["bayesian"], ["optimization", "inference", "learning"]],
        [markov, "decision"],
        [markov, "chain"],
        [[markov], ["decision", "chain"]],
        [prove],
        ["time", "series"],
        [cluster],
        [f"co{_h_}evolution", f"co{_h_}operation"],
        [[f"co{_h_}evolution"], [f"co{_h_}operation"]],
        [[f"co{_h_}operation"], [f"co{_h_}evolution"]],
        [data_driven],
        [predict],
        [recommend, "system"],
        [distribut, "shift"],
    ]

    converg = "converg(?:e|ence|ent|ed|ing)"
    theor = "theor(?:y|etic|etical|etically)"
    analy = "analy(?:ze|sis|zed|zing)"
    bound = "bound(?:|s)"
    run = "run(?:|ning)"
    keywords_theory = [  # theory
        ["drift", "analysis"],
        ["hitting", "time"],
        [evol, converg],
        [evol, "time"],
        [evol, theor],
        [evol, bound],
        [evol, "complexity"],
        ["swarm", converg],
        ["swarm", "time"],
        ["swarm", theor],
        ["swarm", bound],
        ["swarm", "complexity"],
        ["colony", converg],
        ["colony", "time"],
        ["colony", theor],
        ["colony", bound],
        ["colony", "complexity"],
        ["genetic", converg],
        ["genetic", "time"],
        ["genetic", theor],
        ["genetic", bound],
        ["genetic", "complexity"],
        [analy, converg],
        [analy, "time"],
        [analy, theor],
        [analy, bound],
        [analy, "complexity"],
        [computation, "time"],
        [f"{run} time"],
        ["upper", bound],
        ["lower", bound],
        [[converg], [evol, "swarm", "colony", "genetic", analy]],
        [["time"], [evol, "swarm", "colony", "genetic", analy, "hitting", computation, run]],
        [[theor], [evol, "swarm", "colony", "genetic", analy]],
        [[bound], [evol, "swarm", "colony", "genetic", analy, "upper", "lower"]],
        [["complexity"], [evol, "swarm", "colony", "genetic", analy]],
        [[analy], [converg, "time", theor, bound, "complexity"]],
    ]

    keywords_dict = {
        "EC": keywords_ec,
        "SS": keywords_ss,
        "Multi": keywords_multi,
        "Parallel": keywords_parallel,
        "MO": keywords_mo,
        "ML": keywords_ml,
        "ECML": keywords_ec_ml,
        "Theory": keywords_theory,
    }
    return keywords_dict



import os
import re
from typing import Any, Dict, List, Union

from pyadvtools import standard_path, write_list
from pybibtexer.tools.experiments_base import generate_standard_publisher_abbr_options_dict

from ...main import PandocMdTo
from .generate_html import generate_html_content, generate_html_from_bib_data
from .generate_library import generate_library_by_filters


def preparation(
    path_storage: str,
    path_output: str,
    output_basename: str,
    pub_type: str,
    issue_or_month_flag: Union[str, List[str]] = "current_issue",
    year_flag: Union[str, List[str]] = "current_year",
    options: Dict[str, Any] = {},
):
    """Prepare paths and flags for data generation.

    Args:
        path_storage (str): Path to storage directory.
        path_output (str): Path to output directory.
        output_basename (str): Base name for output files.
        pub_type (str): Type of publication.
        issue_or_month_flag (Union[str, List[str]], optional): Issue or month flag. Defaults to "current_issue".
        year_flag (Union[str, List[str]], optional): Year flag. Defaults to "current_year".
        options (Dict[str, Any], optional): Additional options. Defaults to {}.

    Examples:
        |              | current_issue | current_month | all_months |
        |--------------|---------------|---------------|------------|
        | current_year | YES           | YES           | YES        |
        | all_years    | NO            | NO            | YES        |
        | given_years  | NO            | NO            | YES        |

        given_years = ["2020", "2025"]

    Returns:
        Tuple[str, str, bool]: Returns (path_root, path_output, combine_flag).
    """
    # default settings
    path_storage = standard_path(path_storage)
    path_output = standard_path(path_output)

    # "absolute_path" or "relative_path"
    absolute_or_relative_path = options.get("absolute_or_relative_path", "absolute_path")

    # Create path components
    yy = "-".join(year_flag) if isinstance(year_flag, List) else year_flag
    im = "-".join(issue_or_month_flag) if isinstance(issue_or_month_flag, List) else issue_or_month_flag

    if options.get("early_access", False):
        base_path = os.path.join(output_basename, f"{pub_type.title()}_Early_Access", f"{yy}_{im}")
        path_output = os.path.join(path_output + "_Early_Access", f"{yy}_{im}")
    else:
        base_path = os.path.join(output_basename, f"{pub_type.title()}", f"{yy}_{im}")
        path_output = os.path.join(path_output, f"{yy}_{im}")

    path_root = base_path if absolute_or_relative_path == "absolute_path" else ""

    # Determine combine flag
    b = options.get("early_access", False) and (year_flag != "all_years")
    c = year_flag == "current_year"
    c = c and (not isinstance(issue_or_month_flag, list)) and (issue_or_month_flag != "all_months")
    combine_flag = b or c

    return path_root, path_output, combine_flag


def generate_from_bibs_and_write(
    path_storage: str,
    path_output: str,
    output_basename: str,
    pub_type: str,
    generate_or_combine: str,
    year_flag: Union[str, List[str]] = "current_year",
    issue_or_month_flag: Union[str, List[str]] = "current_issue",
    options: Dict[str, Any] = {},
) -> None:
    """Generate or combine data from bibliographies.

    Args:
        path_storage (str): Path to storage directory.
        path_output (str): Path to output directory.
        output_basename (str): Base name for output files.
        pub_type (str): Type of publication.
        generate_or_combine (str): Either "generate_data" or "combine_data".
        year_flag (Union[str, List[str]], optional): Flag for year selection. Defaults to "current_year".
        issue_or_month_flag (Union[str, List[str]], optional): Flag for issue/month selection. Defaults to "current_issue".
        options (Dict[str, Any], optional): Additional options. Defaults to {}.
    """
    path_root, path_output, combine_flag = preparation(
        path_storage, path_output, output_basename, pub_type, issue_or_month_flag, year_flag, options
    )

    if generate_or_combine == "generate_data":
        publisher_abbr_dict = generate_standard_publisher_abbr_options_dict(path_storage, options)
        for publisher in publisher_abbr_dict:
            pp = os.path.join(path_output, publisher.lower())

            publisher_html_body = []
            # Separate for abbr
            for abbr in publisher_abbr_dict[publisher]:
                print(f"*** Processing {publisher.upper()}: {abbr} ***")
                new_options = publisher_abbr_dict[publisher][abbr]

                # Get bibliography path
                path_abbr = os.path.join(path_storage, publisher.lower(), abbr)
                if isinstance(year_flag, str) and year_flag.isdigit():
                    for root, _, files in os.walk(path_abbr, topdown=True):
                        files = [f for f in files if f.endswith(".bib")]
                        if files := [f for f in files if re.search(f"_{year_flag}.bib", f)]:
                            path_abbr = os.path.join(root, files[0])

                # Generate and process library
                library = generate_library_by_filters(
                    path_abbr, issue_or_month_flag, year_flag, new_options
                )

                # Generate md, tex, pdf, html
                html_body = generate_html_from_bib_data(abbr, library, pp, new_options)
                if combine_flag and html_body:
                    publisher_html_body.extend(html_body + ["\n"])

            # Combine for publisher
            if publisher_html_body:
                html_content = generate_html_content(publisher_html_body[:-1], publisher)
                write_list(html_content, f"{publisher}_all.html", "w", pp, False)

    elif generate_or_combine == "combine_data":
        _combine_data(path_storage, path_root, path_output, combine_flag, options)

    return None


def _combine_data(path_storage, path_root, path_output, combine_flag, options):
    """Combine data from multiple sources.

    Args:
        path_storage: Path to storage directory.
        path_root: Root path for output.
        path_output: Path to output directory.
        combine_flag: Flag indicating whether to combine data.
        options: Configuration options.
    """
    # Compulsory
    options["include_abbr_list"] = []
    options["exclude_abbr_list"] = []
    publisher_abbr_dict = generate_standard_publisher_abbr_options_dict(path_storage, options)
    for publisher in publisher_abbr_dict:
        print(f"*** Combining papers for {publisher.upper()} ***")
        pp = os.path.join(path_output, publisher.lower())
        absolute_path = os.path.join(path_root, publisher) if len(path_root) > 0 else ""

        link = [f"# {publisher.upper()}\n\n"]
        for abbr in publisher_abbr_dict[publisher]:
            if os.path.exists(os.path.join(pp, abbr, f"{abbr}.html")):
                ll = os.path.join(absolute_path, abbr, f"{abbr}.html")
                link.append(f"- [{abbr}]({ll})\n")

        if combine_flag:
            ll = os.path.join(absolute_path, f"{publisher}_all.html")
            link.insert(1, f"- [All Journals]({ll})\n")

        # Process combined content
        if len(link) > 1:
            write_list(link, f"{publisher}_link.md", "w", pp, False)
            PandocMdTo({}).pandoc_md_to_html(pp, pp, f"{publisher}_link.md", f"{publisher}_link.html", True)

        # Clean up
        for name in ["_link"]:
            if os.path.exists(file := os.path.join(pp, f"{publisher}{name}.md")):
                os.remove(file)



import copy
import os
import re
import subprocess
import time
from typing import Dict, List, Optional, Tuple

from pyadvtools import (
    combine_content_in_list,
    delete_empty_lines_last_occur_add_new_line,
    insert_list_in_list,
    read_list,
    substitute_in_list,
    write_list,
)

from ..utils.utils import operate_on_generate_html
from .basic_input import BasicInput


class PandocMdTo(BasicInput):
    """Pandoc markdown to various formats (md, tex, html, pdf).

    Args:
        options (dict): Configuration options.

    Attributes:
        markdown_name (str): Markdown name. Defaults to "multi-markdown".
        markdown_citation (str): Markdown citation format. Defaults to "markdown_mmd".
        columns_in_md (int): Number of columns in markdown. Defaults to 120.
        display_one_line_reference_note (bool): Whether to display one line reference note. Defaults to False.
        cite_flag_in_tex (str): Citation flag in LaTeX. Defaults to "cite".
        add_url_for_basic_dict (bool): Whether to add url for items in basic dict. Defualts to True.
        add_anchor_for_basic_dict (bool): Whether to add anchor for items in basic dict. Defaults to False.
        add_anchor_for_beauty_dict (bool): Whether to add anchor for items in beauty dict. Defaults to False.
        add_anchor_for_complex_dict (bool): Whether to add anchor for items in complex dict. Defaults to False.
    """

    def __init__(self, options: dict) -> None:
        """Initialize PandocMdTo with configuration options.

        Args:
            options (dict): Configuration options.
        """
        super().__init__(options)

        self.markdown_name: str = "multi-markdown"
        self.markdown_citation: str = "markdown_mmd"
        if self.markdown_name == "pandoc-markdown":
            self.markdown_citation = "markdown-citations"

        self.columns_in_md: int = options.get("columns_in_md", 120)
        self.display_one_line_reference_note: bool = options.get("display_one_line_reference_note", False)
        # tex
        self.cite_flag_in_tex: str = options.get("cite_flag_in_tex", "cite")

        self.add_url_for_basic_dict: bool = options.get("add_url_for_basic_dict", True)
        self.add_anchor_for_basic_dict: bool = options.get("add_anchor_for_basic_dict", False)
        self.add_anchor_for_beauty_dict: bool = options.get("add_anchor_for_beauty_dict", False)
        self.add_anchor_for_complex_dict: bool = options.get("add_anchor_for_complex_dict", False)

    def pandoc_md_to_md(
        self, path_bib: str, path_md_one: str, path_md_two: str, name_md_one: Optional[str], name_md_two: Optional[str]
    ) -> List[str]:
        """Convert markdown to markdown using pandoc.

        Args:
            path_bib (str): Path to bibliography file.
            path_md_one (str): Path to source markdown directory.
            path_md_two (str): Path to destination markdown directory.
            name_md_one (Optional[str]): Name of source markdown file.
            name_md_two (Optional[str]): Name of destination markdown file.

        Returns:
            List[str]: List of processed markdown content lines.
        """
        full_one = path_md_one if name_md_one is None else os.path.join(path_md_one, name_md_one)
        full_two = path_md_two if name_md_two is None else os.path.join(path_md_two, name_md_two)
        return self._pandoc_md_to_md(full_one, full_two, path_bib)

    def _pandoc_md_to_md(self, full_md_one: str, full_md_two: str, path_bib: str) -> List[str]:
        """Internal method to convert markdown to markdown using pandoc.

        Args:
            full_md_one (str): Full path to source markdown file.
            full_md_two (str): Full path to destination markdown file.
            path_bib (str): Path to bibliography file.

        Returns:
            List[str]: List of processed markdown content lines.
        """
        if not os.path.exists(path_two := os.path.dirname(full_md_two)):
            os.makedirs(path_two)

        if os.path.exists(self.full_csl_style_pandoc):
            cmd = (
                f"pandoc {full_md_one} -t {self.markdown_citation} "
                f"-o {full_md_two} -M reference-section-title='References' "
                f"--citeproc --bibliography={path_bib} --csl={self.full_csl_style_pandoc} --columns {self.columns_in_md}"
            )
        else:
            cmd = (
                f"pandoc {full_md_one} -t {self.markdown_citation} "
                f"-o {full_md_two} -M reference-section-title='References' "
                f"--citeproc --bibliography={path_bib} --columns {self.columns_in_md}"
            )

        try:
            subprocess.run(cmd.split(), check=True, capture_output=True, text=True)
        except subprocess.CalledProcessError as e:
            print("Pandoc error in pandoc md to md:", e.stderr)

        if not os.path.exists(full_md_two):
            print(f"- pandoc false from md to md: {os.path.basename(full_md_two)}\n")
            return []

        return self._standardize_markdown_content(full_md_two)

    @staticmethod
    def _standardize_markdown_content(full_md: str) -> List[str]:
        regex = re.compile(r"(\s*>*\s*[-+*]+)\s\s\s(.*)")
        for i in range(len(data_list := read_list(full_md, "r"))):
            if mch := regex.match(data_list[i]):
                data_list[i] = data_list[i].replace(mch.group(), mch.group(1) + " " + mch.group(2))
        return data_list

    # for pandoc markdown files to tex files
    def pandoc_md_to_tex(
        self, template_name: str, path_md: str, path_tex: str, name_md: Optional[str], name_tex: Optional[str]
    ) -> List[str]:
        full_one = path_md if name_md is None else os.path.join(path_md, name_md)
        full_two = path_tex if name_tex is None else os.path.join(path_tex, name_tex)
        return self._pandoc_md_to_tex(full_one, full_two, template_name)

    def _pandoc_md_to_tex(self, full_md: str, full_tex: str, template_name: str) -> List[str]:
        """Pandoc."""
        if not os.path.exists(path_tex := os.path.dirname(full_tex)):
            os.makedirs(path_tex)

        if template_name.lower() == "beamer":
            cmd = f"pandoc {full_md} -t beamer -o {full_tex} --from markdown "
        else:
            cmd = f"pandoc {full_md} -o {full_tex} --from markdown "

        try:
            subprocess.run(cmd.split(), check=True, capture_output=True, text=True)
        except subprocess.CalledProcessError as e:
            print("Pandoc error in pandoc md to tex:", e.stderr)

        if not os.path.exists(full_tex):
            print(f"- pandoc false from md to tex: {os.path.basename(full_md)}\n")
            return []

        return self._substitute_in_tex_from_md(read_list(full_tex, "r", None))

    def _substitute_in_tex_from_md(self, data_list: List[str]) -> List[str]:
        old_str_list = [r"{\[}@", r"{\[}-@", r"{\[}", r"{\]}", r"\\_"]
        new_str_list = [rf"\\{self.cite_flag_in_tex}" + "{", rf"\\{self.cite_flag_in_tex}" + "{", "{", "}", "_"]
        old_str_list.extend([r"\\footnote<.->{", r";", r"@"])
        new_str_list.extend([r"\\footnote{", ",", ""])
        return substitute_in_list(old_str_list, new_str_list, data_list)

    # for pandoc markdown files to html files
    def pandoc_md_to_html(
        self, path_md: str, path_html: str, name_md: Optional[str], name_html: Optional[str], operate: bool = False
    ) -> str:
        full_one = path_md if name_md is None else os.path.join(path_md, name_md)
        full_two = path_html if name_html is None else os.path.join(path_html, name_html)
        return self._pandoc_md_to_html(full_one, full_two, operate)

    @staticmethod
    def _pandoc_md_to_html(full_md: str, full_html: str, operate: bool = False) -> str:
        """Pandoc."""
        if not os.path.exists(path_html := os.path.dirname(full_html)):
            os.makedirs(path_html)

        cmd = f"pandoc {full_md} -o {full_html} --from markdown "

        try:
            subprocess.run(cmd.split(), check=True, capture_output=True, text=True)
        except subprocess.CalledProcessError as e:
            print("Pandoc error in pandoc md to html:", e.stderr)

        if not os.path.exists(full_html):
            return f"- pandoc false from md to html: {os.path.basename(full_md)}\n"

        if operate:
            operate_on_generate_html(full_html)
        return ""

    # for pandoc markdown files to pdf files
    def pandoc_md_to_pdf(self, path_md: str, path_pdf: str, name_md: Optional[str], name_pdf: Optional[str]) -> str:
        full_one = path_md if name_md is None else os.path.join(path_md, name_md)
        full_two = path_pdf if name_pdf is None else os.path.join(path_pdf, name_pdf)
        return self._pandoc_md_to_pdf(full_one, full_two)

    def _pandoc_md_to_pdf(self, full_md: str, full_pdf: str) -> str:
        """Pandoc."""
        if not os.path.exists(path_pdf := os.path.dirname(full_pdf)):
            os.makedirs(path_pdf)

        if os.path.exists(self.full_tex_article_template_pandoc):
            cmd = (
                f"pandoc {full_md} -o {full_pdf} --from markdown "
                f"--template {self.full_tex_article_template_pandoc} --listings --pdf-engine=xelatex"
            )
        else:
            cmd = f"pandoc {full_md} -o {full_pdf} --from markdown  --listings --pdf-engine=xelatex"

        try:
            subprocess.run(cmd.split(), check=True, capture_output=True, text=True)
        except subprocess.CalledProcessError as e:
            print("Pandoc error in pandoc md to pdf:", e.stderr)

        if not os.path.exists(full_pdf):
            return f"- pandoc false from md to pdf: {os.path.basename(full_md)}\n"
        return ""

    # --------- --------- --------- --------- --------- --------- --------- --------- --------- #
    # md
    def generate_key_data_dict(
        self, pandoc_md_data_list: List[str], key_url_http_bib_dict: Dict[str, List[List[str]]]
    ) -> Tuple[Dict[str, List[str]], Dict[str, List[str]], Dict[str, List[str]]]:
        """Generate."""
        key_reference_dict = self._generate_citation_key_reference_dict_from_pandoc_md(pandoc_md_data_list)
        (key_basic_dict, key_beauty_dict, key_complex_dict) = self._generate_basic_beauty_complex_dict(
            key_url_http_bib_dict, key_reference_dict
        )
        return key_basic_dict, key_beauty_dict, key_complex_dict

    def _generate_citation_key_reference_dict_from_pandoc_md(
        self, pandoc_md_data_list: List[str]
    ) -> Dict[str, List[str]]:
        """Generate."""
        pandoc_md_data_list = self.__append_pandoc_md_reference_part(pandoc_md_data_list)

        line_index, len_data = 0, len(pandoc_md_data_list)
        if self.markdown_name == "pandoc-markdown":
            a_flag, b_flag = r"^:::\s{#ref-(.*)\s\.csl-entry}", r"^:::"
        else:  # multi-markdown
            a_flag, b_flag = r'^<div\sid="ref-(.*?)"', r"^</div>"

        regex_one, regex_two = re.compile(a_flag), re.compile(b_flag)
        key_reference_dict = {}
        while line_index < len_data:
            match_one = regex_one.search(pandoc_md_data_list[line_index])
            line_index += 1
            if not match_one:
                continue
            citation_key = match_one.group(1).strip()

            content = []
            while line_index < len_data:
                line = pandoc_md_data_list[line_index]
                if regex_two.search(line):
                    break

                line_index += 1
                if mch := re.search(r"([<\[]https?://)", line):
                    content.append(line.split(mch.group(1))[0][:-1])
                    break
                else:
                    if line.strip():
                        content.append(line)

            key_reference_dict.update({citation_key: delete_empty_lines_last_occur_add_new_line(content)})
        return key_reference_dict

    def __append_pandoc_md_reference_part(self, pandoc_md_data_list: List[str]) -> List[str]:
        """Append the line which starts with '::: {#'."""
        line_index, len_data = 0, len(pandoc_md_data_list)
        if self.markdown_name == "pandoc-markdown":
            a_flag, b_flag = r":::\s{#", r"}"
        else:  # multi-markdown
            a_flag, b_flag = r'<div\sid="ref', r">"
        regex = re.compile(a_flag)
        new_list = []
        while line_index < len_data:
            line = pandoc_md_data_list[line_index]
            line_index += 1
            if regex.search(line):
                while line_index < len_data:
                    if b_flag != line.rstrip()[-1]:
                        line = line.rstrip() + " " + pandoc_md_data_list[line_index].lstrip()
                        line_index += 1
                    else:
                        new_list.append(line)
                        break
                if line_index == len_data:
                    new_list.append(line)
            else:
                new_list.append(line)
        return delete_empty_lines_last_occur_add_new_line(new_list)

    def _generate_basic_beauty_complex_dict(
        self, key_url_http_bib_dict: Dict[str, List[List[str]]], key_reference_dict: Dict[str, list]
    ) -> Tuple[Dict[str, List[str]], Dict[str, List[str]], Dict[str, List[str]]]:
        """Generate."""
        header_list = ["<details>\n", "```\n"]
        tail_list = ["```\n", "</details>\n"]

        key_basic_dict: Dict[str, List[str]] = {}
        key_beauty_dict: Dict[str, List[str]] = {}
        key_complex_dict: Dict[str, List[str]] = {}

        key_list_http = list(key_url_http_bib_dict.keys())
        key_list_md = list(key_reference_dict.keys())
        if set(key_list_http) != set(key_list_md):
            print(f"`key_list_md`: set({key_list_md}) should be the same with `key_list_http`: set({key_list_http}).")
            return key_basic_dict, key_beauty_dict, key_complex_dict

        for k in key_list_http:
            a: List[str] = copy.deepcopy(key_reference_dict[k])
            b: List[str] = copy.deepcopy(key_reference_dict[k])
            aa: List[str] = key_url_http_bib_dict[k][0]
            bb: List[str] = key_url_http_bib_dict[k][1]

            # add url
            if self.add_url_for_basic_dict:
                a.extend(aa)

            b.extend(bb)

            # add anchor
            if self.add_anchor_for_basic_dict:
                a = [f'<a id="{k.lower()}"></a>\n'] + a
            if self.add_anchor_for_beauty_dict or self.add_anchor_for_complex_dict:
                b = [f'<a id="{k.lower()}"></a>\n'] + b

            if self.display_one_line_reference_note:
                a = ["".join(a).replace("\n", " ").strip() + "\n"]
                b = ["".join(b).replace("\n", " ").strip() + "\n"]

            c = combine_content_in_list([b, header_list, key_url_http_bib_dict[k][2], tail_list])
            key_basic_dict.update({k: a})
            key_beauty_dict.update({k: b})
            key_complex_dict.update({k: c})
        return key_basic_dict, key_beauty_dict, key_complex_dict

    # --------- --------- --------- --------- --------- --------- --------- --------- --------- #
    # tex
    def generate_tex_content(self, file_prefix: str, path_subsection: str, path_bib: str, path_combine: str) -> None:
        # for latex
        # add template for tex file
        add_bib_name = os.path.join(os.path.basename(path_bib), f"{file_prefix}-abbr.bib")
        add_tex_name = os.path.join(os.path.basename(path_subsection), f"{file_prefix}.tex")
        insert_part_one, insert_part_two = self._generate_tex_content(file_prefix, add_tex_name, add_bib_name)

        # write tex
        data_temp = insert_list_in_list(self.article_template_tex, insert_part_one, r"\\begin{document}", "before")
        data_temp = insert_list_in_list(data_temp, insert_part_two, r"\\begin{document}", "after")
        write_list(data_temp, f"{file_prefix}.tex", "w", os.path.join(path_combine, "tex"))
        return None

    @staticmethod
    def _generate_tex_content(file_prefix: str, add_tex_name: str, add_bib_name: str) -> Tuple[List[str], List[str]]:
        if len(file_prefix) == 0:
            file_prefix = "file_prefix"

        part_one = [
            "\\date{" + "{}".format(time.strftime("%B %d, %Y")) + "}\n\n",
            "\\ifx \\fullpath \\undefined\n",
            "    \\addbibresource{" + os.path.join("..", add_bib_name) + "}\n",
            "\\else\n",
            "    \\addbibresource{" + os.path.join("\\fullpath", add_bib_name) + "}\n",
            "\\fi\n\n",
        ]
        part_two = [
            "\n",
            "% \\maketitle\n",
            "\\tableofcontents\n\n",
            "\\ifx \\fullpath \\undefined\n",
            "    \\input{" + os.path.join("..", add_tex_name) + "}\n",
            "\\else\n",
            "    \\input{" + os.path.join("\\fullpath", add_tex_name) + "}\n",
            "\\fi\n",
        ]
        return part_one, part_two



import os

from pyadvtools import GitAutoCommitter

from pyeasyphd.tools import PyRunBibMdTex


def run_article_tex_submit(
    path_input_file: str,
    input_file_names: list[str],
    path_output_file: str,
    bib_path_or_file: str,
    path_conf_j_jsons: str,
    options: dict
) -> None:
    """
    Process academic article files (TeX, and bibliography) with automated Git version control.

    This function handles the conversion and processing of academic article files including TeX documents, and
    bibliography management with automatic Git commit and push capabilities.

    Note: The raw figures and TeX source files must be located in the data/raw subdirectory of the input path.

    Args:
        path_input_file (str): Path to input files directory
        input_file_names (list[str]): List of input file names
        path_output_file (str): Path to output directory
        bib_path_or_file (str): Path to bibliography file or directory
        path_conf_j_jsons (str): Path to conferences and journals JSON files directory
        options (dict): Additional options to override default settings

    Returns:
        None
    """
    path_input_file = os.path.expandvars(os.path.expanduser(path_input_file))
    path_output_file = os.path.expandvars(os.path.expanduser(path_output_file))

    # Initialize default options with detailed descriptions
    _options = {
        "full_json_c": os.path.expanduser(os.path.join(path_conf_j_jsons, "conferences.json")),
        "full_json_j": os.path.expanduser(os.path.join(path_conf_j_jsons, "journals.json")),

        # figure options
        "includegraphics_figs_directory": os.path.join(path_input_file, "data", "raw"),
        "shutil_includegraphics_figs": True,
        "includegraphics_figs_in_relative_path": True,
        "figure_folder_name": "figs",  # "" or "figs" or "main"

        # bib options
        "abbr_index_article_for_abbr": 1,  # 0, 1, 2
        "abbr_index_inproceedings_for_abbr": 0,  # 0, 1, 2
        "add_link_to_fields_for_abbr": None,  # None, or ["title", "journal", "booktitle"]
        "maximum_authors_for_abbr": 0,  # 0, 1, 2, ...
        "add_index_to_entries": False,
        "bib_for_abbr_name": "abbr.bib",
        "bib_for_zotero_name": "zotero.bib",
        "bib_for_save_name": "save.bib",
        "display_google_connected_scite": ["google", "connected", "scite"],

        "bib_folder_name": "bibs",  # "" or "bib" or "bibs" or "main"
        "delete_original_bib_in_output_folder": False,
        "bib_path_or_file": os.path.expanduser(bib_path_or_file),

        # tex options
        "handly_preamble": True,
        "final_output_main_tex_name": "main.tex",
        "run_latex": False,
        "delete_run_latex_cache": False,

        "input_texs_directory": os.path.join(path_input_file, "data", "raw"),
        "shutil_input_texs": True,
        "input_texs_in_relative_path": True,
        "tex_folder_name": "texs",  # "" or "tex" or "texs" or "main"
        "delete_original_tex_in_output_folder": True,
        "generate_tex": True,

        # html options
        "generate_html": False,
    }

    # Update with user-provided options
    _options.update(options)

    committer = GitAutoCommitter(path_output_file)

    if not committer.auto_check(remote="origin", branch="master"):
        print("Remote != Local, please manually pull")

    else:
        # Create full file paths from input file names
        file_list = [os.path.join(path_input_file, f) for f in input_file_names]

        PyRunBibMdTex(path_output_file, ".tex", "paper", _options).run_files(file_list, "", "current")

        if committer.has_changes():
            # Auto commit
            committer.auto_commit()
            # Auto push
            committer.auto_push(remote="origin", branch="master")

    return None



import copy
import os
import re
import shutil
from typing import Any, Dict, List, Tuple

from pyadvtools import (
    IterateCombineExtendDict,
    IterateUpdateDict,
    combine_content_in_list,
    pairwise_combine_in_list,
    read_list,
    sort_int_str,
    standard_path,
    write_list,
)
from pybibtexer.bib.bibtexparser import Library
from pybibtexer.main import PythonRunBib

from ...main import BasicInput
from .search_base import SearchInitialResult
from .search_writers import WriteAbbrCombinedResults
from .utils import keywords_type_for_title, switch_keywords_list, switch_keywords_type


class SearchResultsCore(BasicInput):
    """Core class for generating tex, md, html, and pdf from search results.

    Args:
        path_storage (str): Path to storage directory for bibliography files.
        path_output (str): Path to output directory for generated files.
        path_separate (str): Path to separate directory for individual results.
        j_conf_abbr (str): Abbreviation of journal or conference.
        options (dict): Configuration options.

    Attributes:
        path_storage (str): Path to storage directory.
        path_output (str): Path to output directory.
        path_separate (str): Path to separate directory.
        j_conf_abbr (str): Abbreviation of journal or conference.
        is_standard_bib_file_name (bool): Whether the bib file name follows standard format.
        keywords_type_list (List[str]): List of keyword types to search.
        keywords_dict (dict): Dictionary of keywords for searching.
        delete_redundant_files (bool): Whether to delete redundant files after processing.
        generate_basic_md (bool): Whether to generate basic markdown files.
        generate_beauty_md (bool): Whether to generate beautiful markdown files.
        generate_complex_md (bool): Whether to generate complex markdown files.
        generate_tex (bool): Whether to generate LaTeX files.
        first_field_second_keywords (bool): Whether to search fields first, then keywords.
        deepcopy_library_for_every_field (bool): Whether to deep copy library for every field.
        deepcopy_library_for_every_keywords (bool): Whether to deep copy library for every keywords.
    """

    def __init__(
        self, path_storage: str, path_output: str, path_separate: str, j_conf_abbr: str, options: Dict[str, Any]
    ) -> None:
        """Initialize SearchResultsCore with paths and configuration.

        Args:
            path_storage (str): Path to storage directory for bibliography files.
            path_output (str): Path to output directory for generated files.
            path_separate (str): Path to separate directory for individual results.
            j_conf_abbr (str): Abbreviation of journal or conference.
            options (Dict[str, Any]): Configuration options.
        """
        super().__init__(options)
        self.path_storage: str = standard_path(path_storage)
        self.path_output: str = standard_path(path_output)
        self.path_separate: str = standard_path(path_separate)
        self.j_conf_abbr: str = j_conf_abbr

        # for bib
        # Whether the bib file name is standard, such as `TEVC_2023.bib`.
        self.is_standard_bib_file_name: bool = options.get("is_standard_bib_file_name", True)  # TEVC_2023.bib

        # for search
        self.keywords_dict = options.get("default_keywords_dict", {})
        if temp := options.get("keywords_dict", []):
            self.keywords_dict = temp

        if keywords_type_list := options.get("keywords_type_list", []):
            self.keywords_dict = {k: v for k, v in self.keywords_dict.items() if k in keywords_type_list}

        self.keywords_dict = {switch_keywords_type(k): v for k, v in self.keywords_dict.items()}

        self.search_field_list = options.get("default_search_field_list", ["title", "abstract"])
        if temp := options.get("search_field_list", []):
            self.search_field_list = temp

        # for pandoc
        self.delete_redundant_files: bool = options.get("delete_redundant_files", True)

        # for md
        self.generate_basic_md: bool = options.get("generate_basic_md", False)
        self.generate_beauty_md: bool = options.get("generate_beauty_md", False)
        self.generate_complex_md: bool = options.get("generate_complex_md", True)

        # for tex
        self.generate_tex = options.get("generate_tex", False)

        # for search
        self.first_field_second_keywords = options.get("first_field_second_keywords", True)
        self.deepcopy_library_for_every_field = options.get("deepcopy_library_for_every_field", False)
        self.deepcopy_library_for_every_keywords = options.get("deepcopy_library_for_every_keywords", False)

        # for bib
        self._python_bib = PythonRunBib(options)

    def optimize(self, search_year_list: List[str] = []) -> Dict[str, Dict[str, Dict[str, Dict[str, int]]]]:
        """Optimize search results for given years.

        Args:
            search_year_list (List[str], optional): List of years to search. Defaults to [].

        Returns:
            Dict[str, Dict[str, Dict[str, Dict[str, int]]]]: Nested dictionary containing search results.
        """
        search_year_list = list(set([str(i) for i in search_year_list]))

        data_list = self._obtain_full_files_data(self.path_storage, "bib", search_year_list)

        entry_type_keyword_type_keyword_field_number_dict = self.optimize_core(data_list, search_year_list)
        return entry_type_keyword_type_keyword_field_number_dict

    def _obtain_full_files_data(self, path_storage: str, extension: str, search_year_list: List[str] = []) -> List[str]:
        """Obtain data from all files with specified extension in storage path.

        Args:
            path_storage (str): Path to storage directory.
            extension (str): File extension to search for.
            search_year_list (List[str], optional): List of years to filter by. Defaults to [].

        Returns:
            List[str]: Combined content from all matching files.
        """
        regex = None
        if self.is_standard_bib_file_name and search_year_list:
            regex = re.compile(f'({"|".join(search_year_list)})')

        file_list = []
        for root, _, files in os.walk(path_storage, topdown=True):
            files = [f for f in files if f.endswith(f".{extension}")]

            if regex:
                files = [f for f in files if regex.search(f)]

            file_list.extend([os.path.join(root, f) for f in files])

        return combine_content_in_list([read_list(f, "r") for f in sort_int_str(file_list)], None)

    def optimize_core(self, data_list: List[str], search_year_list) -> Dict[str, Dict[str, Dict[str, Dict[str, int]]]]:
        """Core optimization logic for processing bibliography data.

        Args:
            data_list (List[str]): List of bibliography data strings.
            search_year_list: List of years to search.

        Returns:
            Dict[str, Dict[str, Dict[str, Dict[str, int]]]]: Nested dictionary containing search results.
        """
        print("\n" + "*" * 9 + f" Search in {self.j_conf_abbr} " + "*" * 9)

        entry_type_year_volume_number_month_entry_dict = self._python_bib.parse_to_nested_entries_dict(data_list)

        # generate standard bib and output
        entry_type_keyword_type_keyword_field_number_dict: Dict[str, Dict[str, Dict[str, Dict[str, int]]]] = {}
        for entry_type in entry_type_year_volume_number_month_entry_dict:

            # obtain search years
            year_list = list(entry_type_year_volume_number_month_entry_dict[entry_type].keys())
            if search_year_list:
                year_list = [y for y in year_list if y in search_year_list]
            year_list = sort_int_str(year_list, reverse=True)
            if not year_list:
                print("year_list is empty.")
                continue

            # output prefix
            output_prefix = "-".join([self.j_conf_abbr, year_list[-1], year_list[0]])

            # generate paths
            p_origin = os.path.join(self.path_output, entry_type, f"{output_prefix}-Origin")
            p_separate = os.path.join(self.path_separate, entry_type)
            p_combine = os.path.join(self.path_output, entry_type, f"{output_prefix}-Combine")

            # obtain library
            new_dict = {year: entry_type_year_volume_number_month_entry_dict[entry_type][year] for year in year_list}
            entries = IterateCombineExtendDict().dict_update(new_dict)
            library = Library(entries)

            # search, generate and save
            keyword_type_keyword_field_number_dict = {}
            for keywords_type in self.keywords_dict:
                library = copy.deepcopy(library)

                if self.first_field_second_keywords:
                    keyword_field_number_dict = self._optimize_fields_keyword(
                        keywords_type, library, output_prefix, p_origin, p_separate, p_combine
                    )
                else:
                    keyword_field_number_dict = self._optimize_keywords_field(
                        keywords_type, library, output_prefix, p_origin, p_separate, p_combine
                    )
                keyword_type_keyword_field_number_dict.update({keywords_type: keyword_field_number_dict})

            # collect results
            entry_type_keyword_type_keyword_field_number_dict.setdefault(entry_type, {}).update(
                keyword_type_keyword_field_number_dict
            )

        return entry_type_keyword_type_keyword_field_number_dict

    def _optimize_fields_keyword(self, keywords_type, library, output_prefix, p_origin, p_separate, p_combine):
        """Optimize search by fields first, then keywords.

        Args:
            keywords_type: Type of keywords to search.
            library: Bibliography library to search.
            output_prefix (str): Prefix for output files.
            p_origin (str): Path to origin directory.
            p_separate (str): Path to separate directory.
            p_combine (str): Path to combine directory.

        Returns:
            dict: Dictionary containing keyword field numbers.
        """
        no_search_library = library

        keyword_field_number_dict_ = {}
        for field in self.search_field_list:
            keyword_field_number_dict, no_search_library = self.core_optimize(
                [field], keywords_type, no_search_library, output_prefix, p_origin, p_separate, p_combine
            )

            if self.deepcopy_library_for_every_field:
                no_search_library = copy.deepcopy(library)

            temp = keyword_field_number_dict
            keyword_field_number_dict_ = IterateUpdateDict().dict_update(keyword_field_number_dict_, temp)
        return keyword_field_number_dict_

    def _optimize_keywords_field(self, keywords_type, library, output_prefix, p_origin, p_separate, p_combine):
        """Optimize search by keywords first, then fields.

        Args:
            keywords_type: Type of keywords to search.
            library: Bibliography library to search.
            output_prefix (str): Prefix for output files.
            p_origin (str): Path to origin directory.
            p_separate (str): Path to separate directory.
            p_combine (str): Path to combine directory.

        Returns:
            dict: Dictionary containing keyword field numbers.
        """
        no_search_library = library

        keyword_field_number_dict, no_search_library = self.core_optimize(
            self.search_field_list, keywords_type, no_search_library, output_prefix, p_origin, p_separate, p_combine
        )
        return keyword_field_number_dict

    def core_optimize(
        self,
        search_field_list: List[str],
        keywords_type,
        library: Library,
        output_prefix: str,
        p_origin: str,
        p_separate: str,
        p_combine: str,
    ) -> Tuple[Dict[str, Dict[str, int]], Library]:
        """Core optimization method for processing search results.

        Args:
            search_field_list (List[str]): List of fields to search.
            keywords_type: Type of keywords to search.
            library (Library): Bibliography library to search.
            output_prefix (str): Prefix for output files.
            p_origin (str): Path to origin directory.
            p_separate (str): Path to separate directory.
            p_combine (str): Path to combine directory.

        Returns:
            Tuple[Dict[str, Dict[str, int]], Library]: Tuple containing keyword field numbers and remaining library.
        """
        error_pandoc_md_md: List[str] = []
        save_field_data_dict: Dict[str, List[List[str]]] = {}
        keyword_field_number_dict: Dict[str, Dict[str, int]] = {}

        no_search_library = library
        for keywords_list in self.keywords_dict[keywords_type]:

            print(f"{output_prefix}-{keywords_type}-search-{keywords_list}")
            keywords_list_list, combine_keyword = switch_keywords_list(keywords_list)

            # for initial results
            error_md, field_data_dict, field_number_dict, no_search_library = SearchInitialResult(
                copy.deepcopy(self.options)
            ).main(
                search_field_list,
                p_origin,
                no_search_library,
                keywords_type,
                keywords_list_list,
                combine_keyword,
                output_prefix,
                p_separate,
            )

            if self.deepcopy_library_for_every_keywords:
                no_search_library = copy.deepcopy(library)

            # collect error parts
            error_pandoc_md_md.extend(error_md)

            # collect data
            for field in field_data_dict:
                temp = pairwise_combine_in_list(save_field_data_dict.get(field, []), field_data_dict[field], "\n")
                save_field_data_dict.update({field: temp})

            # collect number
            keyword_field_number_dict.update({combine_keyword: field_number_dict})

        kws_type = keywords_type_for_title(keywords_type)
        flag = "-".join(search_field_list)

        # for error parts in pandoc markdown to markdown
        if error_pandoc_md_md:
            error_pandoc_md_md.insert(0, f"# Error in pandoc md to md for {kws_type}\n\n")
            write_list(error_pandoc_md_md, rf"{flag}_{output_prefix}_error_pandoc_md_md.md", "a", p_combine)

        # combine part
        # for combined results
        error_pandoc_md_pdf, error_pandoc_md_html = WriteAbbrCombinedResults(copy.deepcopy(self.options)).main(
            search_field_list, keywords_type, save_field_data_dict, p_combine
        )

        # for error parts in pandoc markdown to pdf
        if error_pandoc_md_pdf:
            error_pandoc_md_pdf.insert(0, f"# Error in pandoc md to pdf for {kws_type}\n\n")
            write_list(error_pandoc_md_pdf, rf"{flag}_{output_prefix}_error_pandoc_md_pdf.md", "a", p_combine)

        # for error parts in pandoc markdown to html
        if error_pandoc_md_html:
            error_pandoc_md_html.insert(0, f"# Error in pandoc md to html for {kws_type}\n\n")
            write_list(error_pandoc_md_html, rf"{flag}_{output_prefix}_error_pandoc_md_html.md", "a", p_combine)

        # delete redundant files
        if self.delete_redundant_files:
            self.delete_files(keywords_type, p_origin, p_separate, p_combine)

        return keyword_field_number_dict, no_search_library

    def delete_files(self, keywords_type: str, p_origin: str, p_separate: str, p_combine: str) -> None:
        """Delete redundant files after processing.

        Args:
            keywords_type (str): Type of keywords being processed.
            p_origin (str): Path to origin directory.
            p_separate (str): Path to separate directory.
            p_combine (str): Path to combine directory.
        """
        # for initial tex md bib
        if os.path.exists(p_origin):
            shutil.rmtree(p_origin)

        # for separate keywords
        delete_folder_list = []
        if not self.generate_basic_md:
            delete_folder_list.append("basic")
        if not self.generate_beauty_md:
            delete_folder_list.append("beauty")
        if not self.generate_complex_md:
            delete_folder_list.append("complex")

        for d in delete_folder_list:
            for field in self.search_field_list:
                path_delete = os.path.join(p_separate, keywords_type, rf"{field}-md-{d}")
                if os.path.exists(path_delete):
                    shutil.rmtree(path_delete)

        # for combine
        delete_folder_list = ["md"]
        if not self.generate_basic_md:
            delete_folder_list.append("md-basic")
        if not self.generate_beauty_md:
            delete_folder_list.append("md-beauty")
        if not self.generate_complex_md:
            delete_folder_list.append("md-complex")
        if not self.generate_tex:
            delete_folder_list.extend(["tex", "tex-subsection"])

        for d in delete_folder_list:
            path_delete = os.path.join(p_combine, f"{d}")
            if os.path.exists(path_delete):
                shutil.rmtree(path_delete)
